def GetTimeStamp(Split: str = '-') -> str:
    """
    Use To Get TimeStamp

    Args:
        Split (str, '-'): _description_. Used to separate units of time.

    Returns:
        str: _description_. Return a timestamp accurate to the second.
    """
    from time import localtime, strftime
    Time_Local: str = localtime()
    Time_Formatted: str = strftime(f'%Y{Split}%m{Split}%d %H{Split}%M{Split}%S', Time_Local)
    return Time_Formatted

def GetCurrentFrameInfo() -> tuple[str]:  # 获取当前帧信息
    """
    Gets the current code execution location

    Returns:
        tuple[str]: _description_. (Path_File, Name_Func, FuncLine_Def, FuncLine_Current)
    """
    from inspect import currentframe
    # 获取当前栈帧
    CurrentFrame = currentframe()
    # 文件名
    Path_File: str = CurrentFrame.f_code.co_filename
    # 函数名
    Name_Func: str = CurrentFrame.f_code.co_name
    # 函数定义的起始行号
    FuncLine_Def: int = CurrentFrame.f_code.co_firstlineno
    # 当前执行的行号 - 即调用currentframe()的行
    FuncLine_Current: int = CurrentFrame.f_lineno
    return (Path_File, Name_Func, FuncLine_Def, FuncLine_Current)

def lindex(List: list, Value: any) -> int:
    """_用于从左侧查找值索引, 类似 str.find, 如未找到则返回 -1._

    Args:
        List (list): _待查找的列表._
        Value (any): _待查找的值._

    Returns:
        int: _从列表左侧开始找到的第一个索引值._
    """
    if not Value in List: return -1
    Index = List.index(Value)
    return Index

def rindex(List: list, Value: any) -> int:
    """_用于从右侧查找值索引, 类似 str.rfind, 如未找到则返回 -1._

    Args:
        List (list): _待查找的列表._
        Value (any): _待查找的值._

    Returns:
        int: _从列表右侧开始找到的第一个索引值._
    """
    if not Value in List: return -1
    Index = len(List) - 1 - list(reversed(List)).index(Value)
    return Index


class result():
    def __init__(self,
                 state: bool,
                 *args: any,
                 **kwargs: any):
        self.state: bool = state
        self.args: list = list(args)
        self.kwargs: dict = kwargs

    def __bool__(self):
        return self.state
    
    def __getitem__(self, Index: int | str):
        """_Used to obtain the contents of arrays and dictionaries._

        Args:
            Index (int | str): _description_

        Returns:
            _type_: _description_
        """
        if isinstance(Index, int): return self.args[Index]
        if isinstance(Index, str): return self.kwargs[Index]

    def __repr__(self):
        if all((self.args, self.kwargs)) or not any((self.args, self.kwargs)):
            return str(self.args)+str(self.kwargs)
        elif self.args:
            return str(self.args)
        else:
            return str(self.kwargs)

    def append(self, Element: any):
        """_Append Element To result.args._

        Args:
            Element (any): _Any Type._
        """
        self.args.append(Element)

    def update(self, KnV: dict):
            self.kwargs.update(**KnV)



from typing import Literal

def ListFiles(Root: str,
              Types: str | tuple = ('*',),
              Includes: str | tuple = '',
              Mode: Literal['Path', 'Name'] = 'Path') -> list[str]:
    from pathlib import Path
    """_summary_

    Args:
        Root (str): _description_
        Types (Union[str, tuple]): _description_
        Includes (Union[str, tuple], optional): _description_. Defaults to ''.
        Mode (Literal['Path', 'Name'], optional): _description_. Defaults to 'Path'.

    Returns:
        list: _description_
    """
    Types: tuple = (Types,) if isinstance(Types, str) else Types
    Includes: tuple = (Includes,) if isinstance(Includes, str) else Includes
    FileRoot: Path = Path(Root)
    Result: list = []
    for Type in Types: Result += [File.name for File in FileRoot.glob(Type) if all([(Include in File.name) for Include in Includes])]
    if Mode == 'Path': Result = [rf'{Root}\{FileName}' for FileName in Result]
    return Result

def FilesCopyer(Source: str, Target: str, Mode: Literal['Tree', 'File']):
    from os.path import exists as FileExist
    from pathlib import Path
    from shutil import copyfile
    from re import compile
    def IsRepeated(FileName: str):
        if '.' in FileName:
            Match: list = compile(' \(\d+\)\.').findall(FileName)
            if Match:
                Match: str = Match[-1]
                FileName: list = [FileName[:FileName.rfind(Match)],
                                Match[2: -2],
                                FileName[FileName.rfind(Match)+len(Match)-1:],]
            else:
                FileName: list = [FileName[:FileName.rfind('.')],
                                  '',
                                  FileName[FileName.rfind('.'):],]
        else:
            Match: list = compile(' \(\d+\)$').findall(FileName)
            print(Match)
            if Match:
                Match: str = Match[-1]
                FileName: list = [FileName[:FileName.rfind(Match)],
                                  Match[2: -1],
                                  '',]
            else:
                FileName: list = [FileName,
                                  '',
                                  '',]
        return FileName
    if not FileExist(Source): raise Exception('Source File or Source Folder Not Exist')
    if Mode == 'File': pass
    elif Mode == 'Tree': Target = Target + Source[Source.rfind("\\"):]
    Path(Target).mkdir(parents=True, exist_ok=True)

    for FileName in ListFiles(Root=Source, Mode='Name'):
        TargetRootFiles: list = ListFiles(Root=Target, Mode='Name')
        FileNamePart: list = IsRepeated(FileName)
        FileName_Target: str = FileName
        while FileName_Target in TargetRootFiles:
            FileNamePart[1] = 1 if FileNamePart[1] == '' else int(FileNamePart[1])+1
            FileName_Target = f'{FileNamePart[0]} ({FileNamePart[1]}){FileNamePart[2]}'
        FilePath_Target = rf'{Target}\{FileName_Target}'
        FilePath_Source: str = rf'{Source}\{FileName}'
        
        copyfile(FilePath_Source, FilePath_Target)

def FilesClear(*Target: tuple, Mode: Literal['Tree', 'File']):
    from os.path import isfile, isdir
    from os import remove
    from shutil import rmtree
    NewTarget: list = []
    if Mode == 'File':
        for zTarget in Target:
            if isdir(zTarget): NewTarget += ListFiles(zTarget)
            else: NewTarget.append(zTarget)
    for zTarget in NewTarget:
        if isfile(zTarget):
            remove(zTarget)
            continue
        elif isdir(zTarget):
            rmtree(zTarget)
            continue



from typing import Literal

def ASCII_Art(Text: str,
              Font: Literal['univers', 'tarty8', 'tarty7', 'tarty1', 'block'] = 'starwars',
              AddSplit: bool = False) -> str:
    """
    Generate Ascii Art Characters

    Args:
        Text (str): _description_. Source String.
        Font (str, 'univers'): _description_. Set the font for generating Ascii Art.
        AddSplit (bool, False): _description_. Add a context split line

    Returns:
        str: _description_. Ascii Art Characters
    """
    from art import text2art

    ASCIIArt_Str: str = text2art(text=Text, font=Font); SplitLine: str = ''
    if AddSplit: SplitLine: str = '-'*(ASCIIArt_Str.find('\n')) + '\n'
    ASCIIArt_Str: str = SplitLine + ASCIIArt_Str + SplitLine
    LineCount: int = ASCIIArt_Str.count('\n')
    return ASCIIArt_Str, LineCount


def ReadExcel(io: str,
              sheet_name: int = 0,
              KeyTags: list = [],
              SearchRange: int = 15):
    """_用于简化读取Excel文件并进行基础清洗的过程._

    Args:
        io (str): _Excel文件路径_
        sheet_name (int | str, optional): _数据表名称或是索引._ 默认为 0.
        KeyTags (list[list[str]], optional): _数据表列标签中必须包含的关键字, 若单个列标签存在多个可能的关键字则使用列表包裹, 允许缺省._ 默认为 [].
        SearchRange (int, optional): _检索标题行的范围._ 默认为 15, 为 -1 时无限制.

    Raises:
        Exception: _在预设的范围内[header < {TagLine}]无法找到标题行._

    Returns:
        DataFrame: _完成读取与基础清晰的 DataFrame 实例._
    """
    from pandas import DataFrame, read_excel
    from numpy import nan
    Sheet: DataFrame = read_excel(io=io,
                                  sheet_name=sheet_name,
                                  header=None,
                                  dtype=str).fillna('')
    Sheet = Sheet.replace(['', ' '], nan).dropna(how='all').fillna('')
    SearchRange = Sheet.shape[0] if SearchRange == -1 else SearchRange
    KeyTags = [KeyTag if isinstance(KeyTag, list) else [KeyTag] for KeyTag in KeyTags]
    for Index_Row, Row in Sheet.iterrows():
        ColTags = Row.tolist()
        IsTittle: bool = all([any([Tag in ColTags for Tag in KeyTag]) for KeyTag in KeyTags])
        if Index_Row > SearchRange:
            raise Exception(f"在预设的范围内[header < {Index_Row}]无法匹配到标题行.")
        elif IsTittle or not KeyTags:
            Sheet = Sheet.iloc[Index_Row+1:].reset_index(drop=True)
            Sheet.columns = [Tag.strip() if Tag else f'Unnamed: {Index}' for Index, Tag in enumerate(ColTags)]
            return Sheet
    else:
        raise Exception(f"在数据表内无法匹配到标题行.")



import requests
from concurrent.futures import ThreadPoolExecutor
from threading import Lock, Thread
from typing import Literal
from os.path import expanduser, exists
from platform import system
from pylucas import result
from hashlib import sha1, sha256, sha512
from time import time

def HashesVerify(HashMode: Literal['sha1', 'sha256', 'sha512'],
                 File: str,
                 FileHashes: dict):
    match HashMode:
        case 'sha1':
            Hash = sha1()
        case 'sha256':
            Hash = sha256()
        case 'sha512':
            Hash = sha512()
        case _:
            raise Exception('UnSupport Hash Mode.')
    with open(File, "rb") as f:
        for ByteBlock in iter(lambda: f.read(4096), b""):
            Hash.update(ByteBlock)
    if FileHashes[HashMode] == Hash.hexdigest():
        return True
    else:
        return False

class Progress():
    def __init__(self, downloaded: int, total_size: int):
        self.downloaded: int = downloaded
        self.total_size: int = total_size
        self.progress: str = f'{int(downloaded/total_size*10000)/100:.2f}%'

    def __repr__(self):
        return self.progress

    def __getitem__(self, index: Literal['progress', 'downloaded', 'total_size'] | int):
        if isinstance(index, int):
            return (self.progress, self.downloaded, self.total_size)[index]
        elif isinstance(index, str):
            return {"progress": self.progress, "downloaded": self.downloaded, "total_size": self.total_size}[index]

class Downloader(Thread):
    def __init__(self,
                 url: str,
                 FileName: str,
                 FileDir: Literal['Default'] = 'Default',
                 UseThreads: int = 2,
                 FileHashes: dict = {}):
        super().__init__()
        self.name = 'Thread-Downloade'
        self.daemon = True

        self.__Lock: Lock = Lock()
        self.__Progress: dict = {"downloaded": 0,
                                 "total_size": 1}
        self.__FileHashes: dict = FileHashes if any([(Key in FileHashes) for Key in FileHashes]) else {}
        self.__FileHashState: Literal[None, True, False] = None
        self.__DownloadState: Literal[None, True, False] = None
        self.TimeUse: float = 0

        self.url: str = url
        self.FileDir: str = self.DefaultDownloadDir() if FileDir == 'Default' else FileDir
        self.FileName: str = FileName
        self.FilePath: str = rf"{self.FileDir}\{FileName}"
        self.UseThreads: int = UseThreads

        self.FileCheck()
        self.start()

    def __bool__(self):
        return bool(self.__DownloadState)

    @property
    def Progress(self):
        with self.__Lock:
            zProgress: Progress = Progress(self.__Progress["downloaded"], self.__Progress["total_size"])
        return zProgress

    @property
    def FileVerify(self):
        return self.__FileHashState

    def FileCheck(self):
        if not exists(self.FileDir):
            raise FileNotFoundError(f'Folder <{self.FileDir}> Not Found, Make Sure The Dir Exist.')
        FileName: list = [self.FileName[:self.FileName.rfind('.')], self.FileName[self.FileName.rfind('.'):]]
        DupFileNum: int = 0
        
        while exists(self.FilePath):
            DupFileNum += 1
            self.FileName = f"{FileName[0]} ({DupFileNum}) {FileName[1]}"
            self.FilePath = rf"{self.FileDir}\{self.FileName}"
        with open(file=self.FilePath, mode='wb+') as File:
            File.truncate(0)

    def DefaultDownloadDir(self):
        match system():
            case "Windows":
                return f"{expanduser('~')}\Downloads"
            case "Darwin":
                return result(False, 'UnSupport Platfrom.')
            case "Linux":
                return result(False, 'UnSupport Platfrom.')
            case _:
                return result(False, 'UnSupport Platfrom.')

    def run(self):
        self.TimeUse = time()
        super().run()

        zResponse = requests.head(self.url)
        FileSize = int(zResponse.headers['Content-Length'])

        with open(file=self.FilePath, mode='wb+') as File:
            File.truncate(FileSize)

        with self.__Lock:
            self.__Progress["total_size"] = FileSize

        ChunkSize = FileSize // self.UseThreads
        SubChunkSizeBasic: int = 10*8*1024
        SubChunkSize = SubChunkSizeBasic if ChunkSize >= SubChunkSizeBasic * 5 else ChunkSize
        with ThreadPoolExecutor(max_workers=self.UseThreads) as executor:
            futures = []
            for i in range(self.UseThreads):
                Start = i * ChunkSize
                End = Start + ChunkSize - 1 if i != self.UseThreads-1 else FileSize - 1
                futures.append(executor.submit(self.Download_Chunk, Start, End, SubChunkSize))
            for future in futures:
                future.result()
        
        self.TimeUse = f'{(time()-self.TimeUse):.4f}'
        if self.__FileHashes: self.__FileHashState = HashesVerify(HashMode=list(self.__FileHashes.keys())[0],
                                                                  File=self.FilePath,
                                                                  FileHashes=self.__FileHashes)
        self.__DownloadState = self.__FileHashState if self.__FileHashes else True

    def Download_Chunk(self, Start: int, End: int, SubChunkSize: int):
        ChunkDownloaded: int = 0

        headers = {'Range': f'bytes={Start}-{End}'}
        with requests.get(self.url, headers=headers, stream=True) as zResponse:
            zResponse.raise_for_status()
            with open(self.FilePath, 'r+b') as File:
                File.seek(Start)

                for SubChunk in zResponse.iter_content(chunk_size=SubChunkSize):
                    if not SubChunk:
                        continue
                    File.write(SubChunk)
                    ChunkDownloaded += len(SubChunk)
                    if ChunkDownloaded >= SubChunkSize:
                        with self.__Lock: self.__Progress["downloaded"] += ChunkDownloaded
                        ChunkDownloaded = 0

                with self.__Lock: self.__Progress["downloaded"] += ChunkDownloaded

if __name__ == '__main__':
    test = Downloader(url='https://cdn.modrinth.com/data/TQTTVgYE/versions/IRAR0lWa/fabric-carpet-25w05a-1.4.165%2Bv250130.jar',
                    FileName='fabric-carpet-25w05a-1.4.165%2Bv250130.jar',
                    FileDir='Default',
                    UseThreads=1,
                    FileHashes={"sha512": "73443ba8020652ae01bde0a194803cb0ddbde54f0f0f05fa40163aa7e4151dcedfe722cbf2463d99702628e1434c3dc4caf9a0cce799469f20e1ec26f13c49cb",
                                "sha1": "850de39c5e5c0090374963e526d02fc3e4380ce6"})

    from time import sleep

    dobreak = False
    while test.FileVerify == None:
        sleep(0.05)
        print(f'{test.Progress}')
    else:
        print(f'{test.Progress}')
        print(f'Time Total Use: {test.TimeUse}')
        print(f'File Hash State: {test.FileVerify}')
        dobreak = True



from inspect import stack
from os import mkdir
from os.path import exists
from pathlib import Path as _Path
from typing import Literal

from pylucas.function.Function import GetTimeStamp
from pylucas.log.Function import ASCII_Art

class LogManager():
    def __init__(self,
                 Author: str = None,
                 LogConsole: bool = True,
                 LogLimit: int = 10,
                 MsgLineBrk: bool = False,
                 OutPutPath_Root: str = r'.\Log') -> None:
        """_summary_

        Args:
            Author (str, optional): _Print the author's name in the log using AsciiArt._ Defaults to None.
            LogConsole (bool, optional): _Used to set whether to output in the console._ Defaults to True.
            LogLimit (int, optional): _Unlimited: LogLimit<0, Unsaved: LogLimit=0, Limited: LogLimit>0._ Defaults to 10.
            OutPutPath_Root (str, optional): _The root directory to save the log files._ Defaults to r'.\Log'.
        """

        self.Author: str = Author
        self.LogConsole: bool = LogConsole
        self.LogLimit: int = LogLimit

        self.MsgLineBrk: bool = MsgLineBrk
        self.OutPutPath_Root: str = OutPutPath_Root
        self.OutPutPath_File: str = rf'{OutPutPath_Root}\{GetTimeStamp()}.log'

        self.CreateLogFile()
        self.CheckFileLimit()

    def __call__(self,
                 LogMessage: str = 'Invalid Information',
                 Level: Literal['Normal', 'Warn', 'Error'] = 'Normal',
                 Module: str = None,
                 LogConsole: Literal[None, True, False] = None):
        """_summary_

        Args:
            Module (str, optional): _Log Source._ Defaults By Auto Get.
            Level (Literal[Error, Warn, Normal], optional): _Log Level._ Defaults to 'Normal'.
            LogMessage (str, optional): _Log Output Message._ Defaults to 'Invalid Information'.
            LogConsole (bool, optional): _Whether the Log is output in the console._ Defaults is -1 mean fallow to self.LogConsole.
        """
        self.Log(LogMessage=LogMessage, Level=Level, Module=Module, LogConsole=LogConsole)

    def CreateLogFile(self):
        if not self.LogLimit: return

        if not exists(self.OutPutPath_Root): mkdir(self.OutPutPath_Root)
        if self.Author:
            FormatText, LineCount = ASCII_Art(Text=self.Author, AddSplit=True)
            CreatedLog: str = f'Log File Created At {GetTimeStamp()}'+'\n'*(10-(LineCount%10))
        else:
            FormatText: str = ''
            CreatedLog: str = f'Log File Created At {GetTimeStamp()}'+'\n'*10
        with open(file=self.OutPutPath_File, mode='w', encoding='utf-8', ) as LogFile:
            LogFile.write(f'{FormatText}{CreatedLog}')
            LogFile.close()

    def CheckFileLimit(self):
        if self.LogLimit <= 0: return
        Path = _Path(self.OutPutPath_Root)
        Files = [f for f in Path.iterdir() if f.is_file() and f.suffix.lower() == '.log']
        if not Files: return
        while len(Files) > self.LogLimit:
            OldestFile = min(Files, key=lambda f: f.stat().st_mtime)
            OldestFile.unlink()
            self.Log(LogMessage = f'Deleted Oldest LogFile -> {OldestFile}.')
            Files = [f for f in Path.iterdir() if f.is_file() and f.suffix.lower() == '.log']

    def Log(self,
            LogMessage: str = 'Invalid Information',
            Level: Literal['Normal', 'Warn', 'Error'] = 'Normal',
            Module: str = None,
            LogConsole: Literal[None, True, False] = None):
        """_summary_

        Args:
            Module (str, optional): _Log Source._ Defaults By Auto Get.
            Level (Literal[Error, Warn, Normal], optional): _Log Level._ Defaults to 'Normal'.
            LogMessage (str, optional): _Log Output Message._ Defaults to 'Invalid Information'.
            LogConsole (bool, optional): _Whether the Log is output in the console._ Defaults is -1 mean fallow to self.LogConsole.
        """
        TimeStamp: str = GetTimeStamp()
        Level: str = Level
        Module = Module if Module else stack()[1][0].f_globals['__name__']
        Indent: str = '\n\t' if self.MsgLineBrk else ' '
        LogMessage: str = LogMessage[:-1] if LogMessage[-1] in ['.', '。'] else LogMessage

        Message: str = f'{TimeStamp} |-| [Level: <{Level}> | Module: <{Module}>]:{Indent}{LogMessage}.'

        if (self.LogConsole if LogConsole == None else LogConsole): print(Message)

        if not self.LogLimit: return
        with open(file=self.OutPutPath_File, mode='a', encoding='utf-8') as LogFile:
            LogFile.write(f'{Message}\n')
            LogFile.close()





import json, tomllib, tomli_w
from typing import Literal, Self, Any
from copy import deepcopy
from os.path import exists, normpath

class ConfigEditorSL_Toml():
    def Load(self, File: str) -> dict:
        with open(file=File, mode='rb') as File:
            Data = tomllib.load(File)
            File.close()
        return Data

    def Save(self, Data: dict, File: str = '') -> None:
        with open(file=File, mode='wb') as File:
            tomli_w.dump(Data, File)
            File.close()

class ConfigEditorSL_Json():
    def Load(self, File: str) -> dict:
        with open(file=File, mode='r', encoding='utf-8') as File:
            Data = json.load(File)
            File.close()
        return Data

    def Save(self, Data: dict, File: str = '') -> None:
        with open(file=File, mode='w', encoding='utf-8') as File:
            json.dump(Data, File, ensure_ascii=False, indent=4)
            File.close()

class ConfigEditor():
    def __init__(self,
                 File: Literal['<Temporary>'],
                 Data: dict = {}):
        self._Temporary: bool = True if File == '<Temporary>' else False
        self._Flie: str = File if File == '<Temporary>' else normpath(File)
        self._Data: dict = deepcopy(Data)

        self._SetFileMode(FileMode=self._Flie.split('.')[-1].upper() if not self._Temporary else 'TOML')

        if not self._Temporary:
            if exists(File): self.Load()
            else: self.Save(File)

    def _SetFileMode(self, FileMode: Literal['TOML', 'JSON']) -> None:
        match FileMode:
            case 'TOML':
                self._SL = ConfigEditorSL_Toml()
            case 'JSON':
                self._SL = ConfigEditorSL_Json()
            case _ : raise Exception('Target File Mode Not Support.')

    def Load(self):
        if self._Temporary: return
        self._Data = self._SL.Load(self._Flie)

    def Save(self, File: str = '') -> None:
        if File:
            if not isinstance(File, str): raise TypeError(f'Expected str type, get {type(File).__name__}')
            FilePath: str = normpath(File)
            FileRoot: str = FilePath[:FilePath.rfind('\\')]
            FileMode: str = FilePath.split('.')[-1].upper()
            self._SetFileMode(FileMode=FileMode)
            if not exists(path=FileRoot):
                with open(file=FilePath, mode='w', encoding='utf-8') as File: File.close()
            self._Temporary = False
            self._Flie = File
        elif self._Temporary: return

        self._SL.Save(Data=self._Data, File=self._Flie)

    # Pair --------------------------------------------------
    """_下面的方法中的原变量和新变量不会存在任何引用关系._"""
    def __getitem__(self, KeyLocate: str) -> Any:
        return deepcopy(self.GetValue(KeyLocate=KeyLocate, ResultType='Self'))

    def __repr__(self):
        return str(self.ToDict)
    @property
    def ToDict(self) -> dict:
        return deepcopy(self._Data)

    def DataCover(self, Data: dict) -> None:
        """_强制使用深拷贝覆写 self._Data, 不建议使用这个方法._

        Args:
            Data (dict): _description_
        """
        self._Data = deepcopy(Data)
        self.Save()

    # Pair --------------------------------------------------

    def GetKeys(self, KeyLocate: str = '') -> tuple:
        KeyLocate: list = KeyLocate.split('.')
        TempData: Any = self._Data
        Keys: tuple = ()
        if KeyLocate == ['']: KeyLocate = []
        for TempKey in KeyLocate:
            if not isinstance(TempData, dict): raise KeyError(f'{KeyLocate} -> {TempKey}')
            TempData = TempData[TempKey]
        if isinstance(TempData, dict): Keys = tuple(TempData.keys())
        else: Keys = ()

        return Keys

    def POPKey(self, KeyLocate: str) -> None:
        KeyLocate: list = KeyLocate.split('.')
        TempData: Any = self._Data
        for TempKey in KeyLocate[:-1]:
            if not isinstance(TempData, dict): raise KeyError(f'{KeyLocate} -> {TempKey}')
            TempData = TempData[TempKey]
        TempData.pop(KeyLocate[-1])
        self.Save()

    # Pair --------------------------------------------------
    """_下面的方法中的原变量和新变量仍存在引用关系._"""
    def __setitem__(self, KeyLocate: str, Value: Any):
        self.SetValue(KeyLocate=KeyLocate, Value=Value)
        self.Save()

    def GetValue(self,
                 KeyLocate: str,
                 ResultType: Literal['ConfigEditor', 'Self'] = 'ConfigEditor') -> Self | Any:
        """_获取键值, 键值与源字典存在引用关系._

        Args:
            KeyLocate (str): _description_
            ResultType (Literal['ConfigEditor', 'Self'], optional): _description_. Defaults to 'ConfigEditor'.

        Raises:
            KeyError: _description_

        Returns:
            ConfigEditor | Any: _description_
        """
        KeyLocate: list = KeyLocate.split('.')
        TempData: Any = self._Data
        for TempKey in KeyLocate:
            if not isinstance(TempData, dict): raise KeyError(f'{KeyLocate} -> {TempKey}')
            TempData = TempData[TempKey]
        if isinstance(TempData, dict) and ResultType == 'ConfigEditor':
            TempData: ConfigEditor = ConfigEditor(File='<Temporary>', Data=TempData)

        return TempData

    def SetValue(self, KeyLocate: str, Value: Any):
        """_写入键值, 键值与目标字典存在引用关系._

        Args:
            KeyLocate (str): _KeyLocate 所指示的键路径可以不存在于 self._Data 中._
            Value (Any): _description_

        Raises:
            TypeError: _description_
        """
        KeyLocate: list = KeyLocate.split('.')
        TempData: Any = self._Data
        for TempKey in KeyLocate[:-1]:
            if isinstance(TempData, dict):
                if TempKey in TempData:
                    TempData = TempData[TempKey]
                else:
                    TempData.update({TempKey: {}})
                    TempData = TempData[TempKey]
            else:
                raise TypeError(f'TempData: {type(TempData)} = {TempData}')
        TempData.update({KeyLocate[-1]: Value})
        self.Save()

    def AddValue(self, KeyLocate: str, Value: Any):
        """_向可迭代键值中添加元素, 元素与目标字典存在引用关系._

        Args:
            KeyLocate (str): _KeyLocate 所指示的键路径必须存在于 self._Data 中._
            Value (Any): _description_

        Raises:
            KeyError: _KeyLocate 所指示的键路径不存在于 self.__Data中._
            TypeError: _\'UnSupport Type\' object Unable to Add Element._
        """
        KeyLocate: list = KeyLocate.split('.')
        TempData: Any = self._Data
        for TempKey in KeyLocate:
            if not isinstance(TempData, dict): raise KeyError(f'{KeyLocate} -> {TempKey}')
            TempData = TempData[TempKey]

        match type(TempData).__name__:
            case 'int':
                raise TypeError('\'int\' object Unable to Add Element')
            case 'float':
                raise TypeError('\'float\' object Unable to Add Element')
            case 'str':
                raise TypeError('\'str\' object Unable to Add Element')
            case 'tuple':
                raise TypeError('\'tuple\' object Unable to Add Element')
            case 'list':
                TempData.append(Value)
            case 'dict':
                if type(Value) != dict: raise TypeError(f'\'dict\' object Unable to Add a Element of {type(Value).__name__}')
                TempData.update(Value)
            case _:
                raise TypeError(f'\'{type(TempData).__name__}\' object UnSupport to AddElement')

        self.Save()

    # Pair --------------------------------------------------

    def GetNestedPaths(self, KeyLocate: str = '') -> list:
        NestedPaths: list = []
        def DepthRecursion(KeyLocate: str):
            Keys: list = self.GetKeys(KeyLocate=KeyLocate)
            for Key in Keys:
                if KeyLocate == '': KeyLocate_Sub: str = f'{Key}'
                else: KeyLocate_Sub: str = f'{KeyLocate}.{Key}'
                SubKeys = self.GetKeys(KeyLocate=KeyLocate_Sub)
                if SubKeys: DepthRecursion(KeyLocate=KeyLocate_Sub)
                else: NestedPaths.append([KeyLocate, Key])
        DepthRecursion(KeyLocate)
        return NestedPaths




import win32gui
from pprint import pprint
import win32api
import win32con
from types import NoneType
from typing import Literal

def Open():
    path = r"D:\_PsPL\MediaPlayer\Text_Player\Notepad3\Notepad3.exe"
    hinst = win32api.ShellExecute(0, "", path, "", "", 1)
    print(hinst)

def GetWindows() -> list:
    Windows: list = []
    def callback(hwnd, hwnd_list):
        if win32gui.IsWindowVisible(hwnd) and win32gui.GetWindowText(hwnd):
            title = win32gui.GetWindowText(hwnd)
            class_name = win32gui.GetClassName(hwnd)
            Windows.append([hwnd, title, class_name])
        return True  # 返回 True 继续枚举
    win32gui.EnumDesktopWindows(None, callback, None)
    return Windows

class Window():
    def __init__(self, hwnd: int) -> None:
        self.hwnd: int = hwnd

        if not self.IsExist: raise Exception('Non-Existent Window')

    @property
    def IsExist(self) -> bool:
        return bool(win32gui.IsWindow(self.hwnd))

    @property
    def WindowTitle(self) -> str:
        return win32gui.GetWindowText(self.hwnd)
    
    @property
    def WindowClassName(self) -> str:
        return win32gui.GetClassName(self.hwnd)
    
    def SetGeometry(self, x: int, y: int, width: int, height: int):
        win32gui.MoveWindow(self.hwnd, x, y, width, height, True)

    def SetForeground(self) -> None:
        win32gui.SetForegroundWindow(self.hwnd)

    def SetWindowHidden(self, Hide: bool | NoneType = None):
        match Hide:
            case True:
                win32gui.ShowWindow(self.hwnd, win32con.SW_HIDE)
            case False:
                win32gui.ShowWindow(self.hwnd, win32con.SW_SHOW)
            case None:
                pass
        return None
    
    def SetWindowSize(self, Mode: Literal['MINIMIZE', 'MAXIMIZE'] | NoneType = None):
        match Mode:
            case 'MINIMIZE':
                win32gui.ShowWindow(self.hwnd, win32con.SW_MINIMIZE)
            case 'MAXIMIZE':
                win32gui.ShowWindow(self.hwnd, win32con.SW_MAXIMIZE)
            case None:
                pass

def get_first_level_children(hwnd_parent):
    children = []
    win32gui.EnumChildWindows(hwnd_parent, lambda hwnd, result: result.append(hwnd), children)
    return children

from pprint import pprint
pprint(GetWindows())
